%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% E_GetTex.txl
%	- take inputs of symbols, complete the layout pass
%
% Copyright (c) 2005 Ling Zhang, R. Zanibbi
% Copyright (c) 2000-2004 R. Zanibbi
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load BST Grammar and Symbol Class definitions.
% in this file, 
% 1)[analyzeLayout], create the list, code in "Layout_Analysis.Rules"
% 2)[filter],[writeBst], remove the confidence values of symbols and write BST(_full).txt
% 3)set combination method, "MUL" or "MIN", then call [getProbList], combine the confidence 
%   value of BST, code in "Layout_Analysis.Rules"	     
% 4)[SingleStepBubbleSort], sort the result, code in "Layout_Analysis.Rules"
% 5)set top "n", then call [get_Top_n], prune and left the top n, code in 
%   "Layout_Analysis.Rules"

include "E_Grammars/E_BST.Grammar"
include "Grammars/Symbol_Classes.Grammar"

include "E_Rules/E_Layout_Analysis.Rules"
include "E_Rules/E_SymbolFunctions.Rules"
include "E_Rules/E_CommandLineArgs.Rules"

function main
	replace [program]
	    P[program]		

	import TXLargs[repeat stringlit]
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Default global settings (configurable from command line)
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	construct EmptyRegions[repeat relation_node]
	    'TLEFT	'{  '}
	    'BLEFT	'{  '}
	    'ABOVE 	'{  '}
	    'BELOW	'{  '}
	    'SUPER	'{  '}
	    'SUBSC  '{  '}
	    'CONTAINS	'{  '}
	    'ULIMIT	'{  '}
	    'LLIMIT     '{  '}

	construct IntermediateDir [stringlit]
		"./"
	
	construct IntermediateFileName[stringlit]
		"BSTList_TreeWeightsOnly.bst"

	construct GenerateOutput [stringlit]
		"False"

	% By default, do not write out intermediate data.
	construct BaseThresholdRatio[number]
		_[+ 1][/ 8]

	construct ThresholdRatio[number]
		_[+ 1][/ 8]

	construct CentroidRatio[number]
		_[+ 1][/ 4]

	construct OriginLocation[stringlit]
		"Top Left"

	construct Version [stringlit]
		"0.2"

	construct HelpRequested[stringlit]
		"False"

	export IntermediateDir
	export IntermediateFileName
	export GenerateOutput
	export BaseThresholdRatio
	export ThresholdRatio
	export CentroidRatio
	export OriginLocation
	export Version
	export HelpRequested
	export EmptyRegions

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Process command line arguments
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	construct Args[repeat stringlit]
		TXLargs[processArguments]
		#ifdef DEBUG
			[showArgs]  %for debugging.
		#endif

	% Re-import these values; they may have changed.
	import OriginLocation
	import HelpRequested

	where 
		HelpRequested[= "False"]

	by
	    P[processFFESexpression] % for an FFES expression.
end function

% write bst tree to bst.txt file (full bst or just bst)
function writeBst FileName[stringlit] 
	match [any]
		P[any]

	import OriginLocation [stringlit]	
	import IntermediateDir [stringlit]
	import Version [stringlit]
	
	construct Title[repeat stringlit]
		OriginLocation[fput FileName]
		Version[fput FileName]
		P[fput FileName]
end function

% filter out the internal confidence values
function filter
	replace [repeat baselineStructureTreeWithProb]
		BST [repeat baselineStructureTreeWithProb]
		
	deconstruct BST	
		Ft [baselineStructureTreeWithProb] Rt [repeat baselineStructureTreeWithProb]
		
	deconstruct Ft	
		N [opt probtree] { B [baselineStructureTree] }
		
	construct BstWithoutProb [baselineStructureTree]
		B [removeProbTree][removeProb]
	
	construct NewBst [baselineStructureTreeWithProb]
		N { BstWithoutProb }
		
	construct NewRest [repeat baselineStructureTreeWithProb]
		Rt [filter]
		
	by
		NewBst NewRest
end function

% remove the [probtree] in the tree for display
rule removeProbTree
	replace $ [opt probtree]
		P [opt probtree]
		
	by	
end rule

% remove the [prob] in the tree for display
rule removeProb
	replace $ [opt prob]
		P [opt prob]
		
	by	
end rule

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions controlling application of analysis.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function processExpression BST [baselineStructureTreeWithProb]
	replace [repeat baselineStructureTreeWithProb]
		P [repeat baselineStructureTreeWithProb]

	construct OPTION [id]
		'MIN		% the option of combination, 'MUL (multiplication) or 'MIN (minimum)

	export OPTION

	construct Top_n [integernumber] %for SingleStepBubbleSort, only keep the top n in the list
		10
	
	export Top_n
	
	construct NewP[repeat baselineStructureTreeWithProb]
		P [analyzeLayout BST]   % Analyze symbol layout (build BST)

	construct NewPWithProb[repeat baselineStructureTreeWithProb]
		_ [getProbList NewP]
		[SingleStepBubbleSort]
%		[get_Top_n]				% prune and leave the top n answers
%		[print]

	by
		NewPWithProb
end function

function processFFESexpression
	% For a single expression in "FFES" .dat format.
	import IntermediateDir[stringlit]
	import IntermediateFileName[stringlit]

	replace [program]
		H[ffesHeader] BST[baselineStructureTreeWithProb]

	construct BSTWithProb [baselineStructureTreeWithProb]
		BST [giveProbOne]
		
	construct FullBstFileName [stringlit]
		"BSTList_AllWeights.bst"
		
	construct FullBstFilePath [stringlit]
		IntermediateDir[+ FullBstFileName]
		
	construct BstFilePath [stringlit]
		IntermediateDir[+ IntermediateFileName]

	construct P [repeat baselineStructureTreeWithProb]
			
	by
		P[processExpression BSTWithProb]
		[writeBst FullBstFilePath]
	    [filter]
	    [writeBst BstFilePath]
end function
