%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% WriteTex.txl
%	- read bst.txt files (create a list of bst trees), 
%   - complete the lexical pass, and write DRACULAE.bst file
%
% Copyright (c) 2005 Ling Zhang, R. Zanibbi
% Copyright (c) 2000-2004 R. Zanibbi
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load BST Grammar and Symbol Class definitions.

include "E_Grammars/E_BST_Tex.Grammar"
include "Grammars/Symbol_Classes.Grammar"

include "Rules/Relation_Lexing.Rules"
include "Rules/TeX_Output.Rules"
include "Rules/TeX_Symbol_List.Rules"
include "Rules/Token_Lexing.Rules"
include "E_Rules/E_Files.Rules"
include "E_Rules/E_Combination.Rules"
include "E_Rules/E_CommandLineArgs.Rules"

function main
	replace [program]
	    P[program]		

	import TXLargs[repeat stringlit]

	construct IntermediateDir [stringlit]
		"./"
	
	construct IntermediateFileName[stringlit]
		"DRACULAE.bst"

% already done: read bst.txt, create a list of bst tree.    
	deconstruct P
		O[original] BST[repeat baselineStructureTreeWithProb]
		
	deconstruct O
		Origin [originalLocation] Ver [version]
		
	construct  OriginLocation [stringlit]
		_ [quote Origin]
		
	construct Version [stringlit]
		_ [quote Ver]
				
	export IntermediateDir
	export IntermediateFileName
	export Version
	export OriginLocation

	construct Args[repeat stringlit]
		TXLargs[processArguments]
		#ifdef DEBUG
			[showArgs]  %for debugging.
		#endif

	construct Empty [repeat latex_file]
	
	construct NewP[program]
		Empty BST
		
	by
	    NewP[writeIntermediateFileHeader IntermediateFileName]
	     [operate]
end function	    
	    
% write DRACULAE.bst file from the list
rule operate	    
	replace [program]
		Lat[repeat latex_file] BST[repeat baselineStructureTreeWithProb]
		
	import IntermediateFileName[stringlit]

	deconstruct BST
		Ft [baselineStructureTreeWithProb] Rt[repeat baselineStructureTreeWithProb]
		
	deconstruct Ft
		N[opt number] { B [baselineStructureTree] }
		
	construct WriteNumber [opt number]
		N [appendToFile IntermediateFileName "I. Probability"]
		
	construct Convert [baselineStructureTree]
	    B[appendToFile IntermediateFileName "II. Baseline Structure Tree"]
		 
		 [LexTokens]
		 [appendToFile IntermediateFileName "III. Token Lexed BST"]
		 
	   	 [LexRelations]
		 [appendToFile IntermediateFileName "IV. Relation Lexed BST"]

 	     [createTeX]
%		 [processExpressionList] % for a list of expressions.
 	     
 	construct ToLat [repeat latex_file]	     
		 _ [createLaTeX Convert] % generate a latex file.
 	  
	construct NewLat [repeat latex_file]
 		Lat [.  ToLat] 	

 	by
 		NewLat Rt  
end rule

% for each bst tree, create the corresponding latex format
function createLaTeX BST [baselineStructureTree]
	replace [repeat latex_file]
		L [repeat latex_file]
		
	construct DocType[latex_doc]
		'\documentclass '[ '12pt ', 'letter '] '{ 'article '} 
		
	construct FontSize[tex_font_size]
		'Huge
		
	construct LatexHeader[latex_header]
		DocType
		'\usepackage{ 'amssymb '}
		'\begin{document}
		'\pagestyle{ 'empty '}
		'\begin{ FontSize '}
		'$

	construct LatexFooter[latex_footer]
	 '$
     '\end{ FontSize '}
	 '\end{document}

	by
		LatexHeader
		BST[unquoteSymbols]
		LatexFooter
end function

rule unquoteSymbols
	replace $ [symbol]
		Symbol[stringlit]

	construct Dud[id]
		'dud

	construct UnquotedSymbol[id]
		Dud[unquote Symbol]

	construct NewSymbol[symbol]
		UnquotedSymbol

	by
		NewSymbol
end rule

%function processUWExpression
%	replace [UW_III_Expression]
%		Label[expressionLabel] BST[baselineStructureTree]
%	by
%		'\bigskip '% Label '\begin{displaymath} BST[processExpression]
%		'\end{displaymath}
%end function
%
%function processExpressionList
%	% For a list of expressions (e.g. generated from UW_III)
%	replace * [repeat UW_III_Expression]
%	 	expression[UW_III_Expression] Rest[repeat UW_III_Expression]
%
%	import IntermediateFileName[stringlit]
%
%	deconstruct expression
%		Label[expressionLabel] BST[baselineStructureTree]
%
%	construct ExpressionLabel[stringlit]
%		_[quote Label]
%		
%	construct Space[stringlit]
%		""
%		
%	construct FileLabel[stringlit]
%		Space[appendToFile IntermediateFileName ExpressionLabel]
%		
%	by
%		expression[processUWExpression] Rest[processExpressionList]
%end function
