//Short file to take a directory and execute the code for the offline FFES conversion
//This code is mainly copied from ffes.cc in the FFES source code.  Better explanations of the code
//is provided there.

#include <iostream>
#include <fstream>
#include <cstring>
#include <tcl.h>
#include <tk.h>
#include "charrec.cc"
#include <util/Multiline.h>	
#include <vector>

static CharRec CR;
static char passedDir[1024]; //JZ a bit of a hack, but it works..


int displayInit( Tcl_Interp *interp )
{
	if(Tcl_Init(interp) == TCL_ERROR)
		return TCL_ERROR;
	if(Tk_Init(interp) == TCL_ERROR)
		return TCL_ERROR;

  char * FFESDir = getenv("FFES_DIR");
  char * InterfaceFile = "/src/Tk_Interface/Interface.tk";
  
  if (FFESDir == NULL) {
	  std::cerr << "ERROR: FFES_DIR needs to be defined in the environment.\n\tIt"
		  << " should point to the base directory of FFES (e.g. <path>/ffes_0.3)\n";
	  exit(1);
  }

  char MainTkInterfaceFile[256] = "";
  strcat(MainTkInterfaceFile,FFESDir);
  strcat(MainTkInterfaceFile,InterfaceFile);

  std::ifstream InterfaceExists(MainTkInterfaceFile);
  if(! InterfaceExists)
  {
	  std::cerr << "ERROR: Could not find the main interface file (Interface.tk) for FFES at:\n"
		   << "    $(FFES_DIR)" << InterfaceFile << "\n\n"
		   << "    Perhaps the FFES_DIR environment variable is incorrectly defined.\n"
		   << "    Currently FFES_DIR is defined as: \n"
		   << "	   " << FFESDir << "\n\n"
		   << "    Please make sure that FFES_DIR points to the base directory of FFES.\n"
		   << "    (e.g. <path>/ffes_0.3)\n";
	  	
	  InterfaceExists.close();
	  exit(1);
  }
  
  InterfaceExists.close();

  Tcl_SetVar(interp, "dirName", passedDir, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_rcFileName", MainTkInterfaceFile, TCL_GLOBAL_ONLY);

  return 0;
}

	
	//Takes the arguments that are passed to the command line
	//and does various things depending on them.
	//Returns true if the program should execute, else returns false (abort program)
bool handleArgs(int argc, char **argv) {
	bool retVal = true;
	if (argc != 2) {
		cout << endl << endl << "Usage: Display directory" << endl;
		retVal = false;
	} else {
		strncpy(passedDir, argv[1], 1023);
	}
	return retVal;
}


int main(int argc, char **argv)
{
	if (handleArgs(argc, argv)) 
		Tk_Main( 1, argv, displayInit); 
	
	return 0;
}
