//Short file to take a directory and execute the code for the offline FFES conversion
//This code is mainly copied from ffes.cc in the FFES source code.  Better explanations of the code
//is provided there.

#include <iostream>
#include <fstream>
#include <cstring>
#include <tcl.h>
#include <tk.h>
#include "charrec.cc"
#include <util/Multiline.h>	
#include <vector>

static CharRec CR;
static char passedDir[1024]; //JZ a bit of a hack, but it works..
static bool saveImages;

//Outputs a single multiline using point data sent in (argv[1-5]) to file in (argv[6])
//argv[1-5] is used by charrec.cc --> convertTclImageDataToMultiline, although not all of them are "really" needed
int outputMultiline(ClientData client_data, Tcl_Interp* interp,
		     int argc, const char *argv[])
{
	if (argc != 7) {
		sprintf(interp->result,"Wrong number of arguments in outputMultiline in Offline.cc");
		return TCL_ERROR;
	}


	fstream fout(argv[6], ios_base::app | ios_base::out);
	if(fout.is_open()) {
		vector<Multiline> MLs = CR.convertTclImageDataToMultilines(argc, argv);
		fout << MLs[0];
		fout.close();
		return TCL_OK;
	} else {
		cout << "ERROR: Cannot open file: " << argv[6];
		return TCL_ERROR;
	}
}



int offlineInit( Tcl_Interp *interp )
{
	if(Tcl_Init(interp) == TCL_ERROR)
		return TCL_ERROR;
	if(Tk_Init(interp) == TCL_ERROR)
		return TCL_ERROR;

  Tcl_CreateCommand( interp, "outputMultiline", outputMultiline, NULL, NULL);

  char * FFESDir = getenv("FFES_DIR");
  char * InterfaceFile = "/src/Image-Utils/Offline.tk";
  
  if (FFESDir == NULL) {
	  std::cerr << "ERROR: FFES_DIR needs to be defined in the environment.\n\tIt"
		  << " should point to the base directory of FFES (e.g. <path>/ffes_0.3)\n";
	  exit(1);
  }

  char MainTkInterfaceFile[256] = "";
  strcat(MainTkInterfaceFile,FFESDir);
  strcat(MainTkInterfaceFile,InterfaceFile);

  // R. Zanibbi, April 24, 2003: adding explicit test for FFES_DIR
  // value, telling user if they need to make a correction.
  std::ifstream InterfaceExists(MainTkInterfaceFile);
  if(! InterfaceExists)
  {
	  std::cerr << "ERROR: Could not find the main interface file (Interface.tk) for FFES at:\n"
		   << "    $(FFES_DIR)" << InterfaceFile << "\n\n"
		   << "    Perhaps the FFES_DIR environment variable is incorrectly defined.\n"
		   << "    Currently FFES_DIR is defined as: \n"
		   << "	   " << FFESDir << "\n\n"
		   << "    Please make sure that FFES_DIR points to the base directory of FFES.\n"
		   << "    (e.g. <path>/ffes_0.3)\n";
	  	
	  InterfaceExists.close();
	  exit(1);
  }
  
  InterfaceExists.close();

  Tcl_SetVar(interp, "passedDir", passedDir, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_rcFileName", MainTkInterfaceFile, TCL_GLOBAL_ONLY);
  if (saveImages)
	  Tcl_SetVar(interp, "saveImages", "1", TCL_GLOBAL_ONLY);
  else
	  Tcl_SetVar(interp, "saveImages", "0", TCL_GLOBAL_ONLY);

  return 0;
}

	
	//Takes the arguments that are passed to the command line
	//and does various things depending on them.
	//Returns true if the program should execute, else returns false (abort program)
bool handleArgs(int argc, char **argv) {
	bool retVal = true;
	if (argc < 2 || argc > 3 || argc == 3 && !strcmp(argv[1],"-image")) {
		cout << endl << endl << "Usage: Offline [-images] directory" << endl;
		cout << "Help: Offline -help" << endl;
		retVal = false;
	} else if (strcmp(argv[1],"-help") == 0) {
		cout << endl << endl << "Help for Offline FFES" << endl
			 << "---------------------" << endl
			 << "Usage: Offline [-images directory]" << endl
			 << "directory is the root directory of where multiline files are being stored." << endl
			 << "This script will go through all directories starting with root directory and" << endl
			 << "will convert the multiline files so they can be used for image strokes."  << endl
			 << "The new multiline files will be saved in the parent directory of the passed directory /imagedata" << endl
			 << "If -images is specified, a copy of all the images will be sent to the " << endl
			 << "parent directory of the passed directory /img" << endl << endl;
		retVal = false;
	} else if (argc == 2) {
		strncpy(passedDir, argv[1], 1023);
		saveImages = false;
	} else {
		strncpy(passedDir, argv[2], 1023);
		saveImages = true;
	}
	return retVal;
}


int main(int argc, char **argv)
{
	if (handleArgs(argc, argv)) 
		Tk_Main( 1, argv, offlineInit); 
	
	return 0;
}
