#include "RecognitionResults.h"
RecognitionResults::RecognitionResults()
{
	maxKey = -1;
}
	
RecognitionResults::RecognitionResults(const int tmpMaxKey)
{
	maxKey = tmpMaxKey;
}

void RecognitionResults::setMaxKey(const int tmpNumStrokes)
{
	maxKey = tmpNumStrokes;

		//Get rid of any old elements that violate the max key
	if(maxKey >= 1)
	{
		for(map<int, NameConfPairs>::iterator iter = recResults.begin();
				iter != recResults.end(); iter++)
		{
			if(!validKey(iter->first))
				recResults.erase(iter);
		}
	}
}

RecognitionResults::~RecognitionResults()
{
	eraseAll();
}

//Inserts an element with the specified values into the recResults map
void RecognitionResults::insert(const string &key, const vector<string> &symNames, const vector<double> &confs)
{
	insert(convertStringKeyToInt(key), symNames, confs);
}

void RecognitionResults::insert(const int key, const vector<string> &symNames, const vector<double> &confs)
{
	if(validKey(key))
	{
		NameConfPairs tempStruct;
		tempStruct.symNames = symNames;
		tempStruct.confs = confs;
		recResults[key] = tempStruct;
	}
}


void RecognitionResults::erase(const string &key)
{
	erase(convertStringKeyToInt(key));
}


void RecognitionResults::erase(const int key)
{
	if(recResults.find(key) != recResults.end())
		recResults.erase(key);
}


void RecognitionResults::eraseAll()
{
	recResults.erase(recResults.begin(), recResults.end());
}


vector<string> RecognitionResults::getNames(const string &key)
{
	return getNames(convertStringKeyToInt(key));
}


vector<string> RecognitionResults::getNames(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
		return recResults[key].symNames;

	vector<string> tmp;
	tmp.push_back("Invalid key");
	return tmp;
}




vector<double> RecognitionResults::getConfidences(const string &key)
{
	return getConfidences(convertStringKeyToInt(key));
}
	

vector<double> RecognitionResults::getConfidences(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
			return recResults[key].confs;
	
	vector<double> tmp;
	tmp.push_back(-1.0);
	return tmp;
}




//This function goes through the map and looks for the segment
//who has the highest confidence for their first (best) result.
//It returns the key for the best segment.
int RecognitionResults::getHighestConfidenceSegment()
{
	double highest = -1;
	int tempKey;
	for(map<int,NameConfPairs>::iterator iter = recResults.begin();
			iter != recResults.end(); iter++)
	{
		if(iter->second.confs[0] > highest)
		{
			highest = iter->second.confs[0];
			tempKey = iter->first;
		}
	}		
	return tempKey;
}

//JZ: Jun 12, 2006
//The >> operator of stringstream automatically
//discards all whitespace.
string RecognitionResults::stripWhiteSpace(const string &str)
{
	istringstream ss(str);
	string word;
	string result;
	while(ss >> word)
		result += word + " ";

	return result;
}


//It takes the key sent as a string with each element separated by whitespace 
//and returns the key as a vector<int>
int RecognitionResults::convertStringKeyToInt(const string &cKey)
{
	int iKey = 0;
	istringstream ss(cKey);
	int curr;
	
	while(ss >> curr)
	{
		iKey += static_cast<int>(pow(2, curr - 1));
	}
	return iKey;
}


//Returns true/false depending on if the key is valid.
//If maxKey is < 1 automatically return true
//else, all elements of the key must be between 1 and maxKey inclusive
bool RecognitionResults::validKey(const string &key)
{
	if(maxKey < 1)
		return true;
	
	istringstream ss(key);
	int current;
	while(ss >> current)
		if(current < 1 || current > maxKey)
			return false;

	return true;
}

bool RecognitionResults::validKey(const int key)
{
	if(maxKey < 1)
		return true;

	return (key >= 1 && key < pow(2, maxKey));
}


void RecognitionResults::outputAllData()
{
	cout << "Now outputting Recognition Results data....\n\n";
	for(map<int, NameConfPairs>::iterator iter = recResults.begin();
			iter != recResults.end(); iter++)
	{
		cout << "Key: " << iter->first;
	//	cout << "Names --- Address(" << &(iter->second.symNames) << ") Values: ";
		cout << "\nNames: ";
		for(unsigned int i = 0; i < iter->second.symNames.size(); i++)
			cout << iter->second.symNames[i] << " ";

//		cout << "\nConfidences --- Address(" << &(iter->second.confs) << ") Values: ";

		cout << "\nConfidences: ";
		for(unsigned int i = 0; i < iter->second.confs.size(); i++)
			cout << iter->second.confs[i] << " ";
		cout << endl << endl;
		
	}
	cout << "\nFinished outputting Recognition Results data....\n" << endl;
}


//Dead Code
/*
vector<string>* RecognitionResults::getNamesAddr(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
		return &(recResults[key].symNames);

	return NULL;
}


vector<string>* RecognitionResults::getNamesAddr(const string &key)
{
	return getNamesAddr(convertStringKeyToInt(key));
}
vector<double>* RecognitionResults::getConfidencesAddr(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
		return &(recResults[key].confs);

	return NULL;
}


vector<double>* RecognitionResults::getConfidencesAddr(const string &key)
{
	return getConfidencesAddr(convertStringKeyToInt(key));
}
double RecognitionResults::getHighestConfidence(const string &key)
{	
	return getHighestConfidence(convertStringKeyToInt(key));
}
	

double RecognitionResults::getHighestConfidence(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
			return recResults[key].confs[0];
	
	vector<double> tmp;
	tmp.push_back(-1.0);
	return tmp;
}
string RecognitionResults::getHighestName(const string &key)
{
	return getHighestName(convertStringKeyToInt(key));
}


string RecognitionResults::getHighestName(const int key)
{
	if(validKey(key) && recResults.find(key) != recResults.end())
		return recResults[key].symNames[0];

	vector<string> tmp;
	tmp.push_back("Invalid key");
	return tmp;
}*/
