/****************************************************************
/RecognitionResults.h
/This class stores the set of recognition results that are taken
/for a given set of strokes.
/
/The map stores the key as an integer.  The integer is a bitmask
/for all of the strokes, with 1 meaning that the stroke is
/included in the recognition results.
/Example: Strokes 1, 3 --> 1 + 4 = 5
/
/The functions for this class also accept the key as a string
/where the strokes are separated by whitespace.  Example: "1 3".
/The class automatically converts the string key to an integer
/before inserting onto the map.
****************************************************************/
#include <map>
#include <string>
#include <sstream>
#include <vector>
#include <iostream>
#include "math.h"

#ifndef __RECOGNITION_RESULTS_H__
#define __RECOGNITION_RESULTS_H__

using std::string;
using std::stringstream;
using std::istringstream;
using std::vector;
using std::map;
using std::cout;
using std::endl;


class RecognitionResults
{
public:
	RecognitionResults();
	RecognitionResults(const int tmpMaxKey);
	void setMaxKey(const int tmpMaxKey);
	~RecognitionResults();
	void insert(const string &key, const vector<string> &symNames, const vector<double> &confs);
	void insert(const int key, const vector<string> &symNames, const vector<double> &confs);
	void erase(const string &key);
	void erase(const int key);
	void eraseAll();
	vector<string> getNames(const string &key);
	vector<string> getNames(const int key);
	vector<double> getConfidences(const string &key);
	vector<double> getConfidences(const int key);
	int getHighestConfidenceSegment();

	void outputAllData();

	//Dead functions
	/*vector<double>* getConfidencesAddr(const int key);
	vector<double>* getConfidencesAddr(const string &key);
	vector<string>* getNamesAddr(const int key);
	vector<string>* getNamesAddr(const string &key);
	double getHighestConfidence(const string &key);
	double getHighestConfidence(const int key);
	string getHighestName(const string &key);
	string getHighestName(const int key);*/

	
private:
	struct NameConfPairs_T {
		vector<string> symNames;
		vector<double> confs;
	};
	typedef struct NameConfPairs_T NameConfPairs;

	map<int, NameConfPairs> recResults;
		//maxKey < 1 means don't check key
		//otherwise key elements must be between 1 and maxKey inclusive
	int maxKey;

	string stripWhiteSpace(const string &key);
	int convertStringKeyToInt(const string &cKey);

	bool validKey(const string &key);
	bool validKey(const int key);
};

#endif
