/***************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __CharRec_h
#define __CharRec_h

// Smartboard includes
#include <util/Vec2.h>		// 2-D vectors
#include <util/Multiline.h>	// multilines
#include <rec/FeatureRecognizer.h>	// the stroke recogniser
#include <util/Box2.h>		// 2-D boxes.

#include "max_strokes.h"
#include "intersect.h"
#include "RecognitionResults.h"
#include "filter.h"

#include <vector>
#include <math.h>

using std::string;
using std::vector;

#define MAX_STROKES_TO_PROCESS_AT_ONCE  13

class CharRec
{
 private:
  FeatureRecognizer   *Recogniser;
  FeatureRecognizer   *ImageRecogniser;
  vector<int>   	   StrokeIDs[MAX_STROKES_TO_PROCESS_AT_ONCE];
  int                  StrokeCount;
  vector<Multiline>    StrokeMultiline;
  int                  MaxStrokeCount;

 private:
  bool MultilinesIntersect(Multiline &M1, Multiline &M2);
	char *Font_File_Base_Directory(void);

 public:
  CharRec();
  ~CharRec();
  string MakeRecognitionResultForTcl(const vector<int> &highestKey, const vector<vector<string> > &names);
  vector<Multiline> convertTclDataToMultilines(int argc, const char *argv[]);
  vector<Multiline> convertTclImageDataToMultilines(int argc, const char *argv[]);
  char* GetSymbolNames();
  void CombineOverlappingMultilines(vector<Multiline>&);
  FeatureRecognizer* getRecogniser();
  FeatureRecognizer* getImageRecogniser();
};


#endif
