#include "segmenter.h"

//This function will return the power set of the set
//from 1 -> upper except for the empty set.
//Each set of the power set is a different string
//and set elements are separated by spaces.
vector<string> Segmenter::generateSegmentStrings(int upper)
{
	stringstream currentSet;
	string setString;
	vector<string> setResults;

		//This is done because pow() is complaining
		//if I use an integer variable, but I do NOT
		//want this function to accept a double.
	double tempUpper = static_cast<double>(upper);
	for(int i = 1; i < pow(2, tempUpper); i++)
	{
		for(double j = 0; j < tempUpper; j++)
		{
			if(i & int(pow(2, j)))
			{
				currentSet <<  j + 1 <<  " ";
			}
		}
		
		setString = currentSet.str();
		setResults.push_back(setString);
		currentSet.str("");
	}
	return setResults;
}


//Converts the indexes which are stored as one string
//into a vector of ints with the help of a stringstream
vector<int> Segmenter::convertSegmentIndexesToInts(string segmentIndexes)
{
	int currentIndex;
	vector<int> indexes;
	stringstream ss(segmentIndexes);

	while(ss >> currentIndex)
		indexes.push_back(currentIndex);

	return indexes;
}


//Combine the strokes given by indexes onto a single multiline
Multiline Segmenter::combineMultilines(vector<int> indexes, vector<Multiline> mlines)
{
	Multiline tempML;
	for(unsigned int i = 0; i < indexes.size(); i++)
	{
			//Add the - 1 because indexes are 1 based
			//and vectors are 0 based
		 tempML += mlines[indexes[i] - 1];
	}
	return tempML;
}


//This function will take one of the strings returned by generateSegmentStrings
//parse the string to get the idnexes of the multilines
//combine the appropriate multilines, and then run the recognizer.
void Segmenter::recognizeSegment(string segmentIndexes, vector<Multiline> multilines, FeatureRecognizer *recObj, Profile &profObj)
{
	vector<int> intIndexes = convertSegmentIndexesToInts(segmentIndexes);
	Multiline combineMLine = combineMultilines(intIndexes, multilines);
	
	recObj->Recognize(combineMLine, profObj);
}


//This function will parse the profile data into a way
//that the RecognitionResults class can handle and insert
//the data into the RecognitionResults
void Segmenter::addRecognitionResult(string segmentIndexes, Profile prof, RecognitionResults &recObj)
{
	vector<string> symNames;
	vector<double> confs;
	for (int num=0; num < prof.NumChoices(); num++) 
	{
		string currentChar = prof.GetSymbol(num)->Name();
	
			//JZ: Jun  8, 2006
			//This is in the original code for this function
			//I believe it's needed because braces will mess up the lindex call in TCL
		if((currentChar == "{") || (currentChar == "}") || 
		   (currentChar == "[") || (currentChar == "]"))
		{
			num--;
			continue;
		}

		symNames.push_back(currentChar);
		confs.push_back(prof.GetProb(num));
	}

	recObj.insert(segmentIndexes, symNames, confs);
}

//This function recognizes all combinations of strokes that are passed in as multilines
//and stores the recognition results in the RecognitionResults object.
void Segmenter::recognizeStrokes(const vector<Multiline> multilines, FeatureRecognizer *recObj, Profile &profObj, RecognitionResults &recognition)
{
	recognition.setMaxKey(multilines.size());
	vector<string> strokeCombinations = generateSegmentStrings(multilines.size());
	for(unsigned int i = 0; i < strokeCombinations.size(); i++)
	{
		recognizeSegment(strokeCombinations[i], multilines, recObj, profObj);
		addRecognitionResult(strokeCombinations[i], profObj, recognition);
	}
}
